#include <gtk/gtkseparatormenuitem.h>
#include <gtk/gtkdialog.h>
#include <gtk/gtkentry.h>
#include <time.h>
#include <string.h>
#include "common.h"

static void run_command(GtkMenuItem *item, const gchar *cmd);
static void real_run_cmd(GtkDialog *dialog, gint response, gpointer user_data);
static void run_cmd_dialog(GtkMenuItem *item, gpointer user_data);
static void run_settings(GtkMenuItem *item, gpointer user_data);

void
move_left_and_draw(GdkPixbuf *pixbuf, guint user, guint nice, guint sys,
		   guchar r, guchar g, guchar b, guint xx, guint w)
{
	int x, y;
	int width, height, rowstride, n_channels;
	guchar *pixels, *p, *p0;

	n_channels = gdk_pixbuf_get_n_channels(pixbuf);

	g_assert(gdk_pixbuf_get_colorspace(pixbuf) == GDK_COLORSPACE_RGB);
	g_assert(gdk_pixbuf_get_bits_per_sample(pixbuf) == 8);
	g_assert(gdk_pixbuf_get_has_alpha(pixbuf));
	g_assert(n_channels == 4);

	width = gdk_pixbuf_get_width(pixbuf);
	height = gdk_pixbuf_get_height(pixbuf);

	rowstride = gdk_pixbuf_get_rowstride(pixbuf);
	pixels = gdk_pixbuf_get_pixels(pixbuf);

	for (y = AREA_Y; y < AREA_Y + AREA_H; y++) {
		for (x = xx; x < xx + w - 1; x++) {
			p = pixels + y * rowstride + x * n_channels;
			p0 = pixels + y * rowstride + (x + 1) * n_channels;

			p[0] = p0[0];
			p[1] = p0[1];
			p[2] = p0[2];
		}
	}

	for (y = AREA_H; y > 0; y--) {
		p = pixels + (y + AREA_Y - 1) * rowstride + (xx +
							     w -
							     1) *
			n_channels;
		if (AREA_H - y == user) {
			r /= 2;
			g /= 2;
			b /= 2;
		}
		if (AREA_H - y == user + sys) {
			r /= 2;
			g /= 2;
			b /= 2;
		}
		if (AREA_H - y == user + sys + nice) {
			r = 0;
			g = 0;
			b = 0;
		}
		p[0] = r;
		p[1] = g;
		p[2] = b;
	}

}

void int_to_color(gint colorval, GdkColor *color)
{
	if (!color) {
		return;
	}

	color->red = ((colorval >> 16) & 0xff) << 8;
	color->green = colorval & (0xff << 8);
	color->blue = (colorval & 0xff) << 8;
}

gint gc_get_int_fall(GConfClient * gcc, const char *key, gint fallback)
{
	GConfValue *value;
	guint retval;

	value = gconf_client_get(gcc, key, NULL);

	if (value && value->type == GCONF_VALUE_INT) {
		retval = gconf_value_get_int(value);
	} else {
		retval = fallback;
	}
	if (value) {
		gconf_value_free(value);
	}

	return retval;
}

void popup_place(GtkMenu *menu, gint *x, gint *y, gboolean *push_in, gpointer user_data) {
	GtkWidget *btn = GTK_WIDGET(user_data);

	gdk_window_get_position(btn->window, x, y);

	*y += btn->allocation.y + btn->allocation.height;
	*x += btn->allocation.x;
}

void append_items(GtkMenuShell *shell, GConfClient *client, osso_context_t *osso)
{
	GSList *l_cmds;
	GSList *l_cmd_nms;
	GSList *iter, *iter_n;
	GtkWidget *item;
	l_cmds = gconf_client_get_list(client,
			GCONF_PATH_COMMANDS,
			GCONF_VALUE_STRING,
			NULL);

	l_cmd_nms = gconf_client_get_list(client,
			GCONF_PATH_COMMAND_NAMES,
			GCONF_VALUE_STRING,
			NULL);

	for (iter = l_cmds, iter_n = l_cmd_nms; iter && iter_n; 
			iter = iter->next, iter_n = iter_n->next) {
		item = gtk_menu_item_new_with_label(
				(gchar *)iter_n->data);
		gtk_menu_shell_append(shell, item);
		g_signal_connect_data(item, "activate",
				G_CALLBACK(run_command),
				g_strdup((gchar *)iter->data),
				(GClosureNotify)g_free,
				0);
	}

	if (l_cmds && l_cmd_nms) {
		GtkWidget *sep = gtk_separator_menu_item_new();
		gtk_menu_shell_append(shell, sep);
	}

	item = gtk_menu_item_new_with_label("Run command...");
	g_signal_connect(item, "activate", G_CALLBACK(run_cmd_dialog),
			NULL);
	gtk_menu_shell_append(shell, item);
	
	item = gtk_menu_item_new_with_label("Settings...");
	g_signal_connect(item, "activate", G_CALLBACK(run_settings),
			osso);
	gtk_menu_shell_append(shell, item);

	g_slist_foreach(l_cmds, (GFunc)g_free, NULL);
	g_slist_foreach(l_cmd_nms, (GFunc)g_free, NULL);

	g_slist_free(l_cmds);
	g_slist_free(l_cmd_nms);
}

static void run_command(GtkMenuItem *item, const gchar *cmd)
{
	(void)item;
	g_spawn_command_line_async(cmd, NULL);
}

static void run_cmd_dialog(GtkMenuItem *item, gpointer user_data)
{
	GtkWidget *dialog;
	GtkWidget *entry;

	dialog = gtk_dialog_new_with_buttons("Run command",
			NULL,
			GTK_DIALOG_MODAL,
			"Ok", GTK_RESPONSE_ACCEPT,
			"Cancel", GTK_RESPONSE_REJECT,
			NULL);

	entry = gtk_entry_new();
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),
			entry);

	gtk_widget_show_all(dialog);

	g_signal_connect(dialog, "response", G_CALLBACK(real_run_cmd), entry);
}

static void real_run_cmd(GtkDialog *dialog, gint response, gpointer user_data)
{
	GtkEntry *entry = GTK_ENTRY(user_data);
	const gchar *cmd;

	if (response != GTK_RESPONSE_ACCEPT) {
		gtk_widget_destroy(GTK_WIDGET(dialog));
		return;
	}

	cmd = gtk_entry_get_text(entry);

	if (cmd != NULL) {
		g_spawn_command_line_async(cmd, NULL);
	}

	gtk_widget_destroy(GTK_WIDGET(dialog));
}

void clock_overlay(GtkWidget *image) {
	PangoLayout *layout;
	PangoFontDescription *fd;
	const PangoFontDescription *old_fd;
	gchar buffer[128];
	time_t curtime = time(NULL);
	struct tm mytime;

	g_unsetenv("TZ");
	tzset();
	localtime_r(&curtime, &mytime);

	buffer[127] = 0;
	strftime(buffer, 127, "%H:%M", &mytime);

	layout = gtk_widget_create_pango_layout(image, buffer);

	old_fd = pango_layout_get_font_description(layout);
	if (old_fd == NULL) {
		old_fd = pango_context_get_font_description(
				pango_layout_get_context(layout));
	}
	fd = pango_font_description_copy(old_fd);
	pango_font_description_set_absolute_size(fd, 12 * PANGO_SCALE);
	pango_layout_set_font_description(layout, fd);
	pango_font_description_free(fd);
	pango_layout_set_alignment(layout, PANGO_ALIGN_CENTER);
	pango_layout_set_width(layout, AREA_W);

	gdk_draw_layout(GDK_DRAWABLE(image->window),
			image->style->fg_gc[0],
			AREA_X + image->allocation.x + (AREA_W >> 1) - 1,
			AREA_Y + ((AREA_W - 12) >> 1) + image->allocation.y,
			layout);
	gdk_draw_layout(GDK_DRAWABLE(image->window),
			image->style->fg_gc[0],
			AREA_X + image->allocation.x + (AREA_W >> 1) + 1,
			AREA_Y + ((AREA_W - 12) >> 1) + image->allocation.y,
			layout);
	gdk_draw_layout(GDK_DRAWABLE(image->window),
			image->style->fg_gc[0],
			AREA_X + image->allocation.x + (AREA_W >> 1),
			AREA_Y + ((AREA_W - 12) >> 1) + image->allocation.y - 1,
			layout);
	gdk_draw_layout(GDK_DRAWABLE(image->window),
			image->style->fg_gc[0],
			AREA_X + image->allocation.x + (AREA_W >> 1),
			AREA_Y + ((AREA_W - 12) >> 1) + image->allocation.y + 1,
			layout);
	gdk_draw_layout(GDK_DRAWABLE(image->window),
			image->style->bg_gc[0],
			AREA_X + image->allocation.x + (AREA_W >> 1),
			AREA_Y + ((AREA_W - 12) >> 1) + image->allocation.y,
			layout);

	g_object_unref(layout);
}

static void run_settings(GtkMenuItem *item, gpointer user_data)
{
	osso_cp_plugin_execute((osso_context_t *)user_data, "libgraph.so", NULL, TRUE);
}
