#include <glib.h>
#include <gtk/gtk.h>
#include <hildon-widgets/hildon-color-button.h>
#include <hildon-widgets/hildon-caption.h>

#include <libosso.h>
#include <osso-log.h>

#include <assert.h>

#include "osso-applet-graph.h"
#include "common.h"

#define NAME "Device"

enum GraphResponse {
	GRAPH_RESPONSE_COMMANDS = 1,
	GRAPH_RESPONSE_NEW,
	GRAPH_RESPONSE_EDIT,
	GRAPH_RESPONSE_DELETE,
	GRAPH_RESPONSE_DONE
};

static void ui_create_main_dialog(GraphApplet *applet, gpointer window);
static void ui_destroy(GraphApplet *applet);
static void show_cpu_toggled(GtkWidget *widget, GraphApplet *applet);
static void show_mem_toggled(GtkWidget *widget, GraphApplet *applet);
static void combo_toggled(GtkWidget *widget, GraphApplet *applet);
static void clock_toggled(GtkWidget *widget, GraphApplet *applet);
static void mem_color_clicked(GtkWidget *widget, GraphApplet *applet);
static void cpu_color_clicked(GtkWidget *widget, GraphApplet *applet);
static void commands_dialog_response(GtkDialog *dialog, gint response,
	       	GraphApplet *applet);
static gboolean command_dialog_run(GtkWindow *parent, gchar **title, gchar **command);
static void update_cmds(GtkTreeModel *model, GConfClient *gcc);
static void selection_changed(GtkTreeSelection *sel, GraphApplet *applet);
static void move_up(GtkButton *button, GraphApplet *applet);
static void move_down(GtkButton *button, GraphApplet *applet);

osso_return_t execute(osso_context_t * osso, gpointer data,
		      gboolean user_activated)
{
	GraphApplet applet;
	gint response = GRAPH_RESPONSE_COMMANDS;

/*	(void) bindtextdomain(PACKAGE, LOCALEDIR); */

	/* Create the main dialog and refresh the ui */

	ui_create_main_dialog(&applet, data);

	applet.osso = osso;

	if (applet.dialog == NULL) {
		ULOG_CRIT_L("Unable to create applet dialog, exiting..");
		return OSSO_ERROR;
	}

	while (response == GRAPH_RESPONSE_COMMANDS) {
		response = gtk_dialog_run(GTK_DIALOG(applet.dialog));
		if (response == GRAPH_RESPONSE_COMMANDS) {
			gtk_widget_show_all(GTK_WIDGET(applet.commands_dialog));
			gtk_dialog_run(applet.commands_dialog);
			gtk_widget_hide(GTK_WIDGET(applet.commands_dialog));
		}
	}

	if (response == GTK_RESPONSE_ACCEPT) {
		gconf_client_set_bool(applet.gcc, GCONF_PATH_CLOCK_ON,
				applet.flags & CLOCK ? TRUE : FALSE, NULL);
		if (applet.flags & COMBO) {
			gconf_client_set_bool(applet.gcc, GCONF_PATH_CPU_ON,
					FALSE, NULL);
			gconf_client_set_bool(applet.gcc, GCONF_PATH_MEM_ON,
					FALSE, NULL);
			gconf_client_set_bool(applet.gcc, GCONF_PATH_COMBO_ON,
					TRUE, NULL);
			osso_statusbar_send_event(applet.osso, "cpu",
					1, 0, "", NULL);
			osso_statusbar_send_event(applet.osso, "mem",
					1, 0, "", NULL);
			osso_statusbar_send_event(applet.osso, "combo",
					1, 1, "", NULL);
		} else {
			gconf_client_set_bool(applet.gcc, GCONF_PATH_COMBO_ON,
					FALSE,
					NULL);
			gconf_client_set_bool(applet.gcc, GCONF_PATH_CPU_ON,
					(applet.flags & CPU_ON ? TRUE : FALSE),
					NULL);
			gconf_client_set_bool(applet.gcc, GCONF_PATH_MEM_ON,
					(applet.flags & MEM_ON ? TRUE : FALSE),
					NULL);
			osso_statusbar_send_event(applet.osso, "cpu",
					1, (applet.flags & CPU_ON ? 1 : 0),
				       	"", NULL);
			osso_statusbar_send_event(applet.osso, "mem",
					1, (applet.flags & MEM_ON ? 1 : 0),
				       	"", NULL);
			osso_statusbar_send_event(applet.osso, "combo",
					1, 0, "", NULL);
		}
		gconf_client_set_int(applet.gcc, GCONF_PATH_CPU_COLOR,
				applet.cpu_color, NULL);
		gconf_client_set_int(applet.gcc, GCONF_PATH_MEM_COLOR,
				applet.mem_color, NULL);
		update_cmds(gtk_tree_view_get_model(GTK_TREE_VIEW(applet.commands_list)),
			       	applet.gcc);
	}

	/* Make un-initializations */
	ui_destroy(&applet);

	/* Otherwise, return error */
	return OSSO_OK;
}

static void ui_create_main_dialog(GraphApplet *applet, gpointer window)
{
	GtkWidget *dialog_alignment;
	GtkWidget *dialog_vbox;

	GtkWidget *show_cpu;
	GtkWidget *show_mem;

	GtkWidget *cpu_caption;
	GtkWidget *mem_caption;

	GtkWidget *cpu_color;
	GtkWidget *mem_color;

	GtkWidget *cpu_color_caption;
	GtkWidget *mem_color_caption;

	gboolean b_show_cpu;
	gboolean b_show_mem;
	gboolean b_combine;
	gboolean b_clock;

	GSList *l_cmds;
	GSList *l_cmd_nms;
	GSList *iter, *iter_n;

	GtkListStore *model;
	GtkCellRenderer *rend;
	GtkTreeViewColumn *col;
	GtkWidget *view_hbox;
	GtkWidget *view_vbox;

	GtkSizeGroup *size_group;

	GdkColor color;

	applet->flags = 0;
	applet->gcc = gconf_client_get_default();

	b_show_cpu =
		gconf_client_get_bool(applet->gcc, GCONF_PATH_CPU_ON,
				      NULL);

	applet->cpu_color = gc_get_int_fall(applet->gcc,
				      GCONF_PATH_CPU_COLOR, CPU_DEFAULT);

	b_show_mem =
		gconf_client_get_bool(applet->gcc, GCONF_PATH_MEM_ON,
				      NULL);

	applet->mem_color = gc_get_int_fall(applet->gcc,
				      GCONF_PATH_MEM_COLOR, MEM_DEFAULT);

	b_combine = gconf_client_get_bool(applet->gcc,
			GCONF_PATH_COMBO_ON, NULL);

	b_clock = gconf_client_get_bool(applet->gcc,
			GCONF_PATH_CLOCK_ON, NULL);

	l_cmds = gconf_client_get_list(applet->gcc,
			GCONF_PATH_COMMANDS,
			GCONF_VALUE_STRING,
			NULL);

	l_cmd_nms = gconf_client_get_list(applet->gcc,
			GCONF_PATH_COMMAND_NAMES,
			GCONF_VALUE_STRING,
			NULL);

	if (b_combine) {
		b_show_cpu = TRUE;
		b_show_mem = TRUE;
	}
	applet->flags = b_combine * COMBO |
		b_show_cpu * CPU_ON |
		b_show_mem * MEM_ON |
		b_clock * CLOCK;

	applet->dialog =
		GTK_DIALOG(gtk_dialog_new_with_buttons
			   (_("Show graphs"), GTK_WINDOW(window),
			    GTK_DIALOG_MODAL,
			    _("Ok"), GTK_RESPONSE_ACCEPT,
			    _("Commands"), GRAPH_RESPONSE_COMMANDS,
			    _("Cancel"), GTK_RESPONSE_REJECT,
			    NULL)
		);

	dialog_alignment = gtk_alignment_new(0.5, 0.5, 0.0, 0.0);
	dialog_vbox = gtk_vbox_new(FALSE, 0);

	gtk_container_add(GTK_CONTAINER(dialog_alignment), dialog_vbox);
	gtk_container_add(GTK_CONTAINER(applet->dialog->vbox), dialog_alignment);

	size_group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);

	show_cpu = gtk_check_button_new();

	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(show_cpu),
				     b_show_cpu);

	cpu_caption = hildon_caption_new(size_group, _("Show CPU usage"),
			show_cpu, NULL, HILDON_CAPTION_OPTIONAL);

	cpu_color = hildon_color_button_new();

	int_to_color(applet->cpu_color, &color);

	hildon_color_button_set_color(HILDON_COLOR_BUTTON(cpu_color), &color);

	cpu_color_caption = hildon_caption_new(size_group, _("Graph colour"),
			cpu_color, NULL, HILDON_CAPTION_OPTIONAL);

	g_signal_connect_after(G_OBJECT(show_cpu),
			       "toggled", G_CALLBACK(show_cpu_toggled),
			       applet);

	g_signal_connect_after(G_OBJECT(cpu_color),
			       "clicked", G_CALLBACK(cpu_color_clicked),
			       applet);

	show_mem = gtk_check_button_new();

	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(show_mem),
				     b_show_mem);

	mem_caption = hildon_caption_new(size_group, _("Show mem usage"),
			show_mem, NULL, HILDON_CAPTION_OPTIONAL);

	mem_color = hildon_color_button_new();

	int_to_color(applet->mem_color, &color);

	hildon_color_button_set_color(HILDON_COLOR_BUTTON
				      (mem_color), &color);

	mem_color_caption = hildon_caption_new(size_group, _("Graph colour"),
			mem_color, NULL, HILDON_CAPTION_OPTIONAL);

	g_signal_connect_after(G_OBJECT(show_mem),
			       "toggled", G_CALLBACK(show_mem_toggled),
			       applet);

	g_signal_connect_after(G_OBJECT(mem_color),
			       "clicked", G_CALLBACK(mem_color_clicked),
			       applet);

	gtk_container_add(GTK_CONTAINER(dialog_vbox),
			  GTK_WIDGET(cpu_caption)
		);
	gtk_container_add(GTK_CONTAINER(dialog_vbox),
			  GTK_WIDGET(cpu_color_caption)
		);

	gtk_container_add(GTK_CONTAINER(dialog_vbox),
			  GTK_WIDGET(mem_caption)
		);
	gtk_container_add(GTK_CONTAINER(dialog_vbox),
			  GTK_WIDGET(mem_color_caption)
		);

	applet->combo_check = gtk_check_button_new();
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(applet->combo_check),
				     b_combine);
	applet->combo_caption = hildon_caption_new(size_group,
			_("Combine graphs"), applet->combo_check, NULL,
			HILDON_CAPTION_OPTIONAL);

	applet->clock_check = gtk_check_button_new();
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(applet->clock_check),
				     b_clock);
	applet->clock_caption = hildon_caption_new(size_group,
			_("Show clock"), applet->clock_check, NULL,
			HILDON_CAPTION_OPTIONAL);

	gtk_widget_set_sensitive(applet->combo_caption,
		       	         b_show_cpu && b_show_mem);
	gtk_widget_set_sensitive(applet->clock_caption,
		       	         b_show_cpu || b_show_mem);

	g_signal_connect_after(G_OBJECT(applet->combo_check),
			       "toggled", G_CALLBACK(combo_toggled),
			       applet);

	g_signal_connect_after(G_OBJECT(applet->clock_check),
			       "toggled", G_CALLBACK(clock_toggled),
			       applet);

	gtk_container_add(GTK_CONTAINER(dialog_vbox),
			  GTK_WIDGET(applet->combo_caption)
		);
	gtk_container_add(GTK_CONTAINER(dialog_vbox),
			  GTK_WIDGET(applet->clock_caption)
		);

	applet->commands_dialog = GTK_DIALOG(
			gtk_dialog_new_with_buttons(_("Commands"),
			GTK_WINDOW(applet->dialog),
			GTK_DIALOG_MODAL | GTK_DIALOG_NO_SEPARATOR,
			_("New"), GRAPH_RESPONSE_NEW,
			_("Edit"), GRAPH_RESPONSE_EDIT,
			_("Delete"), GRAPH_RESPONSE_DELETE,
			_("Done"), GRAPH_RESPONSE_DONE,
			NULL));

	model = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_STRING);

	for (iter = l_cmds, iter_n = l_cmd_nms; iter && iter_n; 
			iter = iter->next, iter_n = iter_n->next) {
		GtkTreeIter titer;
		gtk_list_store_append(model, &titer);
		gtk_list_store_set(model, &titer, 0, iter_n->data,
				1, iter->data, -1);
	}

	applet->commands_list = gtk_tree_view_new_with_model(GTK_TREE_MODEL(model));
	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(applet->commands_list), FALSE);
	g_signal_connect(gtk_tree_view_get_selection(GTK_TREE_VIEW(applet->commands_list)), "changed", G_CALLBACK(selection_changed), applet);
	g_object_unref(model);

	rend = gtk_cell_renderer_text_new();
	col = gtk_tree_view_column_new_with_attributes("Title", rend,
			"text", 0, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(applet->commands_list), col);

	view_hbox = gtk_hbox_new(FALSE, 0);
	view_vbox = gtk_vbox_new(FALSE, 0);

	applet->view_up = gtk_button_new_from_stock(GTK_STOCK_GO_UP);
	applet->view_down = gtk_button_new_from_stock(GTK_STOCK_GO_DOWN);

	g_signal_connect(applet->view_up, "clicked", G_CALLBACK(move_up), applet);
	g_signal_connect(applet->view_down, "clicked", G_CALLBACK(move_down), applet);

	g_slist_foreach(l_cmds, (GFunc)g_free, NULL);
	g_slist_foreach(l_cmd_nms, (GFunc)g_free, NULL);

	g_slist_free(l_cmds);
	g_slist_free(l_cmd_nms);

	gtk_box_pack_start(GTK_BOX(view_hbox), applet->commands_list,
			TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(view_hbox), view_vbox,
			FALSE, FALSE, 0);
	gtk_container_add(GTK_CONTAINER(view_vbox), applet->view_up);
	gtk_container_add(GTK_CONTAINER(view_vbox), applet->view_down);

	gtk_container_add(GTK_CONTAINER(applet->commands_dialog->vbox),
		       	view_hbox);

	g_signal_connect(applet->commands_dialog, "response",
			G_CALLBACK(commands_dialog_response), applet);

	gtk_widget_show_all(GTK_WIDGET(applet->dialog)
		);
}

osso_return_t save_state(osso_context_t * osso, gpointer data)
{
	return OSSO_OK;
}

static void ui_destroy(GraphApplet *applet)
{
	gtk_widget_destroy(GTK_WIDGET(applet->dialog));
	gtk_widget_destroy(GTK_WIDGET(applet->commands_dialog));
}

static void show_cpu_toggled(GtkWidget *widget, GraphApplet *applet)
{
	applet->flags &= ~CPU_ON;
	applet->flags |= gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)) * CPU_ON;
	gconf_client_clear_cache(applet->gcc);
	while (gtk_events_pending())
		gtk_main_iteration();
	if (applet->flags & (CPU_ON | MEM_ON)) {
		gtk_widget_set_sensitive(applet->clock_caption, TRUE);
	}
	else {
		gtk_widget_set_sensitive(applet->clock_caption, FALSE);
	}
	if ((applet->flags & (CPU_ON | MEM_ON)) == (CPU_ON | MEM_ON)) {
		gtk_widget_set_sensitive(applet->combo_caption, TRUE);
	} else {
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(applet->combo_check), FALSE);
		gtk_widget_set_sensitive(applet->combo_caption, FALSE);
	}
}

static void show_mem_toggled(GtkWidget *widget, GraphApplet *applet)
{
	applet->flags &= ~MEM_ON;
	applet->flags |= gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)) * MEM_ON;
	gconf_client_clear_cache(applet->gcc);
	while (gtk_events_pending())
		gtk_main_iteration();
	if (applet->flags & (CPU_ON | MEM_ON)) {
		gtk_widget_set_sensitive(applet->clock_caption, TRUE);
	}
	else {
		gtk_widget_set_sensitive(applet->clock_caption, FALSE);
	}
	if ((applet->flags & (CPU_ON | MEM_ON)) == (CPU_ON | MEM_ON)) {
		gtk_widget_set_sensitive(applet->combo_caption, TRUE);
	} else {
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(applet->combo_check), FALSE);
		gtk_widget_set_sensitive(applet->combo_caption, FALSE);
	}
}

static void combo_toggled(GtkWidget *widget, GraphApplet *applet)
{
	applet->flags &= ~COMBO;
	applet->flags |= gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)) * COMBO;
	gconf_client_clear_cache(applet->gcc);
	while (gtk_events_pending())
		gtk_main_iteration();
}

static void clock_toggled(GtkWidget *widget, GraphApplet *applet)
{
	applet->flags &= ~CLOCK;
	applet->flags |= gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)) * CLOCK;
	gconf_client_clear_cache(applet->gcc);
	while (gtk_events_pending())
		gtk_main_iteration();
}

static void cpu_color_clicked(GtkWidget *widget, GraphApplet *applet)
{
	GdkColor *color =
		hildon_color_button_get_color(HILDON_COLOR_BUTTON(widget)
		);

	applet->cpu_color = (color->red & 0xff00) << 8 |
		(color->green & 0xff00) |
		color->blue >> 8;
}

static void mem_color_clicked(GtkWidget *widget, GraphApplet *applet)
{
	GdkColor *color =
		hildon_color_button_get_color(HILDON_COLOR_BUTTON(widget)
		);

	applet->mem_color = (color->red & 0xff00) << 8 |
		(color->green & 0xff00) |
		color->blue >> 8;
}

static void commands_dialog_response(GtkDialog *dialog, gint response,
		GraphApplet *applet)
{
	GtkTreeIter iter;
	GtkTreeSelection *selection;
	GtkTreeModel *model;
	gchar *title = NULL;
	gchar *command = NULL;
	switch (response) {
		case GRAPH_RESPONSE_DONE:
			return;
			break;
		case GRAPH_RESPONSE_NEW:
			if (command_dialog_run(GTK_WINDOW(applet->commands_dialog), &title, &command)) {
				model = gtk_tree_view_get_model(GTK_TREE_VIEW(applet->commands_list));
				gtk_list_store_append(GTK_LIST_STORE(model), &iter);
				gtk_list_store_set(GTK_LIST_STORE(model),
						&iter,
						0, title,
						1, command,
						-1);
			}
			break;
		case GRAPH_RESPONSE_EDIT:
			selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(applet->commands_list));
			if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
				gtk_tree_model_get(model, &iter, 0, &title,
						1, &command, -1);
				if (command_dialog_run(GTK_WINDOW(applet->commands_dialog), &title, &command)) {
					gtk_list_store_set(GTK_LIST_STORE(model),
							&iter,
							0, title,
							1, command,
							-1);
				}
			}
			break;
		case GRAPH_RESPONSE_DELETE:
			selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(applet->commands_list));
			if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
				gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
			}
			break;
	}
	if (title) g_free(title);
	if (command) g_free(command);
	g_signal_stop_emission_by_name(dialog, "response");
}

static gboolean command_dialog_run(GtkWindow *parent, gchar **title, gchar **command)
{
	GtkWidget *dialog = gtk_dialog_new_with_buttons(_("Edit command"),
			parent,
			GTK_DIALOG_MODAL | GTK_DIALOG_NO_SEPARATOR,
			_("Done"), GTK_RESPONSE_ACCEPT,
			_("Cancel"), GTK_RESPONSE_REJECT,
			NULL);
	GtkWidget *title_entry = gtk_entry_new();
	GtkWidget *command_entry = gtk_entry_new();
	GtkSizeGroup *group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
	GtkWidget *title_caption = hildon_caption_new(group,
			_("Description"),
			title_entry,
			NULL,
			HILDON_CAPTION_OPTIONAL);
	GtkWidget *command_caption = hildon_caption_new(group,
			_("Command"),
			command_entry,
			NULL,
			HILDON_CAPTION_OPTIONAL);

	if (*title) {
		gtk_entry_set_text(GTK_ENTRY(title_entry), *title);
	}
	if (*command) {
		gtk_entry_set_text(GTK_ENTRY(command_entry), *command);
	}

	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),
			title_caption);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),
			command_caption);

	gtk_widget_show_all(dialog);

	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
		g_free(*title);
		*title = g_strdup(gtk_entry_get_text(GTK_ENTRY(title_entry)));
		*command = g_strdup(gtk_entry_get_text(GTK_ENTRY(command_entry)));
		gtk_widget_destroy(dialog);
		return TRUE;
	}
	
	gtk_widget_destroy(dialog);
	return FALSE;
}

static void update_cmds(GtkTreeModel *model, GConfClient *gcc)
{
	GSList *l_cmd = NULL;
	GSList *l_cmd_nm = NULL;
	GtkTreeIter iter;

	if (gtk_tree_model_get_iter_first(model, &iter)) {
		gchar *cmd, *name;

		do {
			gtk_tree_model_get(model, &iter, 0, &name, 1, &cmd, -1);
			l_cmd = g_slist_append(l_cmd, cmd);
			l_cmd_nm = g_slist_append(l_cmd_nm, name);
		} while (gtk_tree_model_iter_next(model, &iter));
	}

	gconf_client_set_list(gcc, GCONF_PATH_COMMAND_NAMES,
			GCONF_VALUE_STRING, l_cmd_nm, NULL);
	gconf_client_set_list(gcc, GCONF_PATH_COMMANDS,
			GCONF_VALUE_STRING, l_cmd, NULL);

	g_slist_foreach(l_cmd, (GFunc)g_free, NULL);
	g_slist_foreach(l_cmd_nm, (GFunc)g_free, NULL);
	g_slist_free(l_cmd);
	g_slist_free(l_cmd_nm);
}

static void selection_changed(GtkTreeSelection *sel, GraphApplet *applet)
{
	GtkTreeModel *model;
	GtkTreeIter iter;

	if (gtk_tree_selection_get_selected(sel, &model, &iter)) {
		GtkTreePath *path = gtk_tree_model_get_path(model, &iter);
		if (gtk_tree_path_prev(path)) {
			gtk_widget_set_sensitive(applet->view_up, TRUE);
		} else {
			gtk_widget_set_sensitive(applet->view_up, FALSE);
		}
		gtk_tree_path_free(path);
		if (gtk_tree_model_iter_next(model, &iter)) {
			gtk_widget_set_sensitive(applet->view_down, TRUE);
		} else {
			gtk_widget_set_sensitive(applet->view_down, FALSE);
		}
	}
}

static void move_up(GtkButton *button, GraphApplet *applet)
{
	GtkTreeIter iter;
	GtkTreeSelection *selection;
	GtkTreeModel *model;

	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(applet->commands_list));

	if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
		GtkTreePath *path = gtk_tree_model_get_path(model, &iter);
		if (gtk_tree_path_prev(path)) {
			GtkTreeIter iter2;
			gtk_tree_model_get_iter(model, &iter2, path);
			gtk_list_store_move_before(GTK_LIST_STORE(model),
					&iter,
					&iter2);
		}
		gtk_tree_path_free(path);
		g_signal_emit_by_name(selection, "changed");
	}
}

static void move_down(GtkButton *button, GraphApplet *applet)
{
	GtkTreeIter iter;
	GtkTreeSelection *selection;
	GtkTreeModel *model;

	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(applet->commands_list));

	if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
		GtkTreeIter iter2 = iter;
		if (gtk_tree_model_iter_next(model, &iter2)) {
			gtk_list_store_move_after(GTK_LIST_STORE(model),
					&iter,
					&iter2);
		}
		g_signal_emit_by_name(selection, "changed");
	}
}
